#!/usr/bin/python
import numpy as np
import math
import matplotlib
matplotlib.use("qt4agg")
from matplotlib import pyplot as plt
import sys
import time
import subprocess
import random
import copy
import MultilineTRLdesign

optfilename = "ML_TRL_runXX.txt"

makefigure = 0
expotempprofile = 0
lintempprofile = 1
expoparamprofile = 1
linparamprofile = 0
verbose = 0
keepordered = 1

#This code performs a simulated annealing search for the optimum CL over a set of parameters. 

random.seed()

#optimization variables
temp = 0.5 #dB - initial temperature in terms of conversion loss. Gives 1/e point for acceptance
totalsteps = 10000
resetfreq = 1000000 #after this many sucessful steps the previous maximum is found

#expo temp profile
if expotempprofile == 1:
	tempdecreaseperthou = 0.4 #fractional decrease in temperature during each step
	tempdecrease = tempdecreaseperthou**(0.001)
if lintempprofile:
	holdathigh = 0.1 #frac of time at initial temp
	holdatlow = 0.02 #frac of time at zero temp
	tempdecrease = temp/(totalsteps*(1-holdathigh-holdatlow))
	
deltadecreaseperthou = 0.75 #determines how delta shrinks with time
deltadecrease = deltadecreaseperthou**(0.001)

#band to average over
freqs = np.linspace(1,50,491)

#swept sim parameters
l1 = 0
l2 = 1
l3 = 3
l4 = 10

#build first param point array
new_paramlist = []
new_paramlist.append(l1)
new_paramlist.append(l2)
new_paramlist.append(l3)
new_paramlist.append(l4)

#parameter ranges
l1_range = [0,1]
l2_range = [0,3]
l3_range = [1,10]
l4_range = [3,15]
ranges = []
ranges.append(l1_range)
ranges.append(l2_range)
ranges.append(l3_range)
ranges.append(l4_range)

#characteristic widths in parameters
delta_l1 = 0
delta_l2 = 0.5
delta_l3 = 1
delta_l4 = 0
deltalist = []
deltalist.append(delta_l1)
deltalist.append(delta_l2)
deltalist.append(delta_l3)
deltalist.append(delta_l4)

#initialize arrays for recording sucesses
paramarray = []
paramarray.append(new_paramlist[:])
optimizearray = []
stepnumarray = []

#initialize arrays for recording all calculated
recparamarray =[]
recparamarray.append(new_paramlist[:])
recoptimizearray = []
recstepnumarray = []

if makefigure > 0:
	#figures for plot
	fig1 = plt.figure(1)
	ax11 = fig1.add_subplot(151)
	ax12 = fig1.add_subplot(152)
	ax13 = fig1.add_subplot(153)
	ax14 = fig1.add_subplot(154)
	ax15 = fig1.add_subplot(155)
	plt.ion()
	
print("Solving Inital Model for:")
print(new_paramlist)
	
#solve model for this point
optimizeparam = MultilineTRLdesign.getTRLCostFunction(new_paramlist)
	
print("Model solved, intial optimization parameter value is: " + str(optimizeparam))

optimizearray.append(optimizeparam)
recoptimizearray.append(optimizeparam)
stepnumarray.append(0)
recstepnumarray.append(0)
prevoptimizeparam = optimizeparam

print("Starting Optimization")
print(paramarray)
print(optimizearray)

stepnum = 1
while stepnum <= totalsteps:
	
	if verbose:
		print("AT STEP # " + str(stepnum) + "/" + str(totalsteps))
	elif stepnum%100 == 0:
		print("AT STEP # " + str(stepnum) + "/" + str(totalsteps))

	#now find new point off of last vector in paramarray
	new_paramlist_ref = paramarray[len(paramarray)-1]
	new_paramlist = new_paramlist_ref[:]
	if verbose:
		print("Starting from this list:")
		print(new_paramlist)
	paramtoupdate = -1
	while paramtoupdate == -1 or deltalist[paramtoupdate] == 0:
		paramtoupdate = random.randint(0,len(new_paramlist)-1)
	if verbose:
		print("Updating param #" + str(paramtoupdate) + " = " + str(new_paramlist[paramtoupdate]))
	oldparam = new_paramlist[paramtoupdate]
	
	#figure out how much the sigma needs to be
	sigmastep = deltalist[paramtoupdate]*(deltadecrease**stepnum)
	#generate new point and check if it's within range
	foundanewparampoint = 0
	while foundanewparampoint == 0:
		new_paramlist[paramtoupdate] = random.gauss(oldparam,sigmastep)
		#print("New param is = " + str(new_paramlist[paramtoupdate]))
		#print("Updated " + str(oldparam) + " to " + str(new_paramlist[paramtoupdate]))
		if new_paramlist[paramtoupdate] >= ranges[paramtoupdate][0] and new_paramlist[paramtoupdate] <= ranges[paramtoupdate][1] and new_paramlist[paramtoupdate] != oldparam:
			if verbose:
				print("New parameter is within range")
			if (not keepordered) or ((new_paramlist[paramtoupdate] > new_paramlist[paramtoupdate-1]) and (new_paramlist[paramtoupdate] < new_paramlist[paramtoupdate+1])):
				foundanewparampoint = 1
				if verbose and keepordered:
					print("New params are still in order")
			elif keepordered:
				if verbose:
					print("Update would disorder parameters")
		else:
			if verbose:
				print("New parameter is outside of range")
	
	if verbose:
		print("Solving model for:")
		print(new_paramlist)
	
	#solve model at this point
	optimizeparam = MultilineTRLdesign.getTRLCostFunction(new_paramlist)
		
	if verbose:
		print("Model solved, optimization parameter value is: " + str(optimizeparam))
	
	#decide if to keep this point or choose another
	delta_opt = optimizeparam - prevoptimizeparam
	if delta_opt > 0  and temp <= 0 :
		if verbose:
			print("The new model is better and accepted at zero temp: " + str(optimizeparam) + " > " + str(prevoptimizeparam))
		paramarray.append(new_paramlist[:])
		prevoptimizeparam = optimizeparam
		optimizearray.append(optimizeparam)
		stepnumarray.append(stepnum)
	elif temp <= 0 and delta_opt <= 0:
		if verbose:
			print("The new model is worse and rejeected at zero temp")
	elif delta_opt > 0 or random.random() < math.exp(delta_opt/temp):
		if delta_opt > 0:
			if verbose:
				print("The new model is better and accepted: " + str(optimizeparam) + " > " + str(prevoptimizeparam))
		else:
			if verbose:
				print("The new model is worse but accepted at temp " + str(temp) +  " : " + str(optimizeparam) + " < " + str(prevoptimizeparam))
		paramarray.append(new_paramlist[:])
		prevoptimizeparam = optimizeparam
		optimizearray.append(optimizeparam)
		stepnumarray.append(stepnum)
	else:
		if verbose:
			print("The new model is rejected: " + str(optimizeparam) + " < " + str(prevoptimizeparam))
	
	#now record all steps for plotting
	recparamarray.append(new_paramlist[:])
	recoptimizearray.append(optimizeparam)
	recstepnumarray.append(stepnum)
	
	if makefigure == 1 or (makefigure == 2 and stepnum%10 == 0):
		ax11.cla()
		ax11.plot(recstepnumarray,recoptimizearray,'-r')
		ax11.plot(stepnumarray,optimizearray,'-b')
		ax11.set_xlabel('Step Number')
		ax11.set_ylabel(r'$1/\sigma_\alpha + 1/\sigma_\beta$')
	
		#ax12.cla()
		ax12.scatter(np.array(recparamarray)[:,0],recoptimizearray)
		ax12.set_xlabel('l1 (mm)')
		ax12.set_ylabel(r'$1/\sigma_\alpha + 1/\sigma_\beta$')
		ax12.set_xlim(ranges[0])
		
		#ax13.cla()
		ax13.scatter(np.array(recparamarray)[:,1],recoptimizearray)
		ax13.set_xlabel('l2 (mm)')
		ax13.set_ylabel(r'$1/\sigma_\alpha + 1/\sigma_\beta$')
		ax13.set_xlim(ranges[1])
		
		#ax14.cla()
		ax14.scatter(np.array(recparamarray)[:,2],recoptimizearray)
		ax14.set_xlabel('l3 (mm)')
		ax14.set_ylabel(r'$1/\sigma_\alpha + 1/\sigma_\beta$')
		ax14.set_xlim(ranges[2])
		
		#ax15.cla()
		ax15.scatter(np.array(recparamarray)[:,3],recoptimizearray)
		ax15.set_xlabel('l4 (mm)')
		ax15.set_ylabel(r'$1/\sigma_\alpha + 1/\sigma_\beta$')
		ax15.set_xlim(ranges[3])
	
		plt.draw()
		plt.pause(0.01)
	
	if len(optimizearray)%resetfreq == 0:
		if verbose:
			print("Reseting to local maximum at step # : " + str(stepnum))
		#reset parameters to the best one in the last resetfreq successful steps
		tempoptimize = optimizearray[(len(optimizearray)-resetfreq-1):len(optimizearray)]
		tempparams = paramarray[(len(optimizearray)-resetfreq-1):len(optimizearray)]
		bestpos = -1
		bestopt = -100
		for pos in range(0,len(tempoptimize)):
			if bestpos == -1 or tempoptimize[pos] > bestopt:
				bestopt = tempoptimize[pos]
				bestpos = pos
		besttempparams = tempparams[bestpos]
		paramarray.append(besttempparams[:])
		if verbose:
			print("resetting to a optimization parameter value of : " + str(bestopt))
		prevoptimizeparam = bestopt
		if verbose:
			print(besttempparams)
		optimizearray.append(bestopt)
		stepnumarray.append(stepnum + 0.5)
		
	
	stepnum = stepnum + 1
	if expotempprofile == 1:
		temp = temp*tempdecrease
	if lintempprofile == 1:
		if stepnum > holdatlow*totalsteps:
			temp = temp - tempdecrease
			if temp < 0:
				temp = 0

if makefigure > 0:
	plt.ioff()
	plt.show()

print("Writing output file to: " + optfilename)

fout = open(optfilename,"w")
for linenum in range(0,len(recoptimizearray)):
	fout.write(str(recparamarray[linenum][0]) + " " + str(recparamarray[linenum][1]) + " " + str(recparamarray[linenum][2]) + " " + str(recparamarray[linenum][3]) + " " + str(recoptimizearray[linenum]) + "\n")	
fout.close()