#!/usr/bin/python
import numpy as np
import math
import matplotlib
matplotlib.use("qt4agg")
from matplotlib import pyplot as plt
import sys
import time
import subprocess
import random
import copy
import MultilineTRLdesign

#Simulated Annealing Optimization Algorithm

def TRLopt(lineparams, freqs, opt, outfilename = "ML_TRL_RunXXX.txt"):

	#lineparams = np.array(lineparams)
	freqs = np.array(freqs)
	#opt = np.array(opt)
	
	expotempprofile = 0     #Exponentially decreasing temperature
	lintempprofile = 1		#Linearly decreasing temperature, better for high dimension problems
	expoparamprofile = 0	#Exponentially decreasing parameter variation
	linearparamprofile = 1  #Linearly decreasing parameter variation
	verbose = 0             #Print EVERYTHING, only do if stuff isn't working
	keepordered = 1         #Keep parameters ordered

	#This code performs a simulated annealing search for the optimum CL over a set of parameters. 
	random.seed()

	#optimization variables
	optname = opt[0]
	temp = opt[1] #dB - initial temperature in terms of conversion loss. Gives 1/e point for acceptance
	totalsteps = opt[2]
	physicaltemp = opt[3]
	resetfreq = 1000000 #after this many sucessful steps the previous maximum is returned to

	#expo temp profile
	if expotempprofile == 1:
		tempdecreaseperthou = 0.4 #fractional decrease in temperature during each step
		tempdecrease = tempdecreaseperthou**(0.001)
	if lintempprofile:
		holdathigh = 0.1 #frac of time at initial temp
		holdatlow = 0.02 #frac of time at zero temp
		tempdecrease = temp/(totalsteps*(1-holdathigh-holdatlow))
	
	if expoparamprofile == 1:
		deltadecreaseperthou = 0.75 #determines how delta shrinks with time
		deltadecrease = deltadecreaseperthou**(0.001)
	elif linearparamprofile == 1:
		paramholdathigh = 0.25
		paramholdatlow = 0.1
		parammin = 0.1
		deltadecrease = (1-parammin)/(totalsteps*(paramholdathigh-paramholdatlow))

	paramnames = []
	new_paramlist = []
	deltalist = []
	ranges = []
	for i in range(0,len(lineparams)):
		paramnames.append(lineparams[i][0])
		new_paramlist.append(lineparams[i][1])
		deltalist.append(lineparams[i][2])
		ranges.append(lineparams[i][3])
	
	if verbose == 1:
		print(paramnames)
		print(new_paramlist)
		print(deltalist)
		print(ranges)
	
	#initialize arrays for recording sucesses
	paramarray = []
	paramarray.append(new_paramlist[:])
	optimizearray = []
	stepnumarray = []

	#initialize arrays for recording all calculated
	recparamarray =[]
	recparamarray.append(new_paramlist[:])
	recoptimizearray = []
	recstepnumarray = []
		
	print("Solving Inital Model for:")
	print(new_paramlist)
	
	#solve model for this point
	optimizeparam = MultilineTRLdesign.getTRLCostFunction(new_paramlist,freqs,physicaltemp)
		
	print("Model solved, intial optimization parameter value is: " + str(optimizeparam))
	
	optimizearray.append(optimizeparam)
	recoptimizearray.append(optimizeparam)
	stepnumarray.append(0)
	recstepnumarray.append(0)
	prevoptimizeparam = optimizeparam

	print("Starting Optimization")

	stepnum = 1
	while stepnum <= totalsteps:
	
		if verbose:
			print("AT STEP # " + str(stepnum) + "/" + str(totalsteps))
		elif stepnum%100 == 0:
			print("AT STEP # " + str(stepnum) + "/" + str(totalsteps))

		#now find new point off of last vector in paramarray
		new_paramlist_ref = paramarray[len(paramarray)-1]
		new_paramlist = new_paramlist_ref[:]
		if verbose:
			print("Starting from this list:")
			print(new_paramlist)
		paramtoupdate = -1
		while paramtoupdate == -1 or deltalist[paramtoupdate] == 0:
			paramtoupdate = random.randint(0,len(new_paramlist)-1)
		if verbose:
			print("Updating param #" + str(paramtoupdate) + " = " + str(new_paramlist[paramtoupdate]))
		oldparam = new_paramlist[paramtoupdate]
	
		#figure out how much the sigma needs to be
		if expoparamprofile == 1:
			sigmastep = deltalist[paramtoupdate]*(deltadecrease**stepnum)
		elif linearparamprofile == 1:
			if stepnum/totalsteps <= paramholdathigh:
				sigmastep = deltalist[paramtoupdate]
			elif stepnum/totalsteps >=(1-paramholdatlow):
				sigmastep = deltalist[paramtoupdate]*parammin
			else:
				sigmastep = deltalist[paramtoupdate]*(1-paramholdatlow-stepnum/totalsteps+parammin*(stepnum/totalsteps-paramholdathigh))/(1-paramholdathigh-paramholdatlow)
			if verbose:
				print("Normalized Step Width is now: " + str(sigmastep/deltalist[paramtoupdate]))
		#generate new point and check if it's within range
		foundanewparampoint = 0
		while foundanewparampoint == 0:
			new_paramlist[paramtoupdate] = random.gauss(oldparam,sigmastep)
			#print("New param is = " + str(new_paramlist[paramtoupdate]))
			#print("Updated " + str(oldparam) + " to " + str(new_paramlist[paramtoupdate]))
			stillordered = 1 #check if list is ordered still
			for paramnum in range(0,len(new_paramlist)-1):
				if new_paramlist[paramnum] > new_paramlist[paramnum+1]:
					stillordered = 0 #not ordered anymore
			if new_paramlist[paramtoupdate] >= ranges[paramtoupdate][0] and new_paramlist[paramtoupdate] <= ranges[paramtoupdate][1] and new_paramlist[paramtoupdate] != oldparam:
				if verbose:
					print("New parameter is within range")
				if (not keepordered) or stillordered:
					foundanewparampoint = 1
					if verbose and keepordered:
						print("New params are still in order")
				elif keepordered:
					if verbose:
						print("Update would disorder parameters")
			else:
				if verbose:
					print("New parameter is outside of range")
			
		
		if verbose:
			print("Solving model for:")
			print(new_paramlist)
		
		#solve model at this point
		optimizeparam = MultilineTRLdesign.getTRLCostFunction(new_paramlist,freqs,physicaltemp)
			
		if verbose:
			print("Model solved, optimization parameter value is: " + str(optimizeparam))
		
		#decide if to keep this point or choose another
		delta_opt = optimizeparam - prevoptimizeparam
		if delta_opt > 0  and temp <= 0 :
			if verbose:
				print("The new model is better and accepted at zero temp: " + str(optimizeparam) + " > " + str(prevoptimizeparam))
			paramarray.append(new_paramlist[:])
			prevoptimizeparam = optimizeparam
			optimizearray.append(optimizeparam)
			stepnumarray.append(stepnum)
		elif temp <= 0 and delta_opt <= 0:
			if verbose:
				print("The new model is worse and rejeected at zero temp")
		elif delta_opt > 0 or random.random() < math.exp(delta_opt/temp):
			if delta_opt > 0:
				if verbose:
					print("The new model is better and accepted: " + str(optimizeparam) + " > " + str(prevoptimizeparam))
			else:
				if verbose:
					print("The new model is worse but accepted at temp " + str(temp) +  " : " + str(optimizeparam) + " < " + str(prevoptimizeparam))
			paramarray.append(new_paramlist[:])
			prevoptimizeparam = optimizeparam
			optimizearray.append(optimizeparam)
			stepnumarray.append(stepnum)
		else:
			if verbose:
				print("The new model is rejected: " + str(optimizeparam) + " < " + str(prevoptimizeparam))
		
		#now record all steps for plotting
		recparamarray.append(new_paramlist[:])
		recoptimizearray.append(optimizeparam)
		recstepnumarray.append(stepnum)
		
		if len(optimizearray)%resetfreq == 0:
			if verbose:
				print("Reseting to local maximum at step # : " + str(stepnum))
			#reset parameters to the best one in the last resetfreq successful steps
			tempoptimize = optimizearray[(len(optimizearray)-resetfreq-1):len(optimizearray)]
			tempparams = paramarray[(len(optimizearray)-resetfreq-1):len(optimizearray)]
			bestpos = -1
			bestopt = -100
			for pos in range(0,len(tempoptimize)):
				if bestpos == -1 or tempoptimize[pos] > bestopt:
					bestopt = tempoptimize[pos]
					bestpos = pos
			besttempparams = tempparams[bestpos]
			paramarray.append(besttempparams[:])
			if verbose:
				print("resetting to a optimization parameter value of : " + str(bestopt))
			prevoptimizeparam = bestopt
			if verbose:
				print(besttempparams)
			optimizearray.append(bestopt)
			stepnumarray.append(stepnum + 0.5)
			
		
		stepnum = stepnum + 1
		if expotempprofile == 1:
			temp = temp*tempdecrease
		if lintempprofile == 1:
			if stepnum > holdatlow*totalsteps:
				temp = temp - tempdecrease
				if temp < 0:
					temp = 0

	print("Writing output file to: " + outfilename)

	fout = open(outfilename,"w")
	for linenum in range(0,len(recoptimizearray)):
		outstr = ""
		for i in range(0,len(recparamarray[linenum])):
			outstr = outstr + str(recparamarray[linenum][i]) + " "
		outstr = outstr + str(recoptimizearray[linenum]) + "\n"
		fout.write(outstr)	
	fout.close()
	
	maxopt = 0
	maxparams = []
	for i in range(0,len(recoptimizearray)):
		if i == 0:
			maxopt = recoptimizearray[0]
			maxparams = recparamarray[0]
		elif recoptimizearray[i] > maxopt:
			maxopt = recoptimizearray[i]
			maxparams = recparamarray[i]
			
	return maxparams
			
		