#!/usr/bin/python
import numpy as np
import math
import matplotlib
from matplotlib import pyplot as plt
import matplotlib.cm as cm
import skrf as rf
import MultilineTRLdesign as TRL

def plotMLTRL(lines,freqs,physicaltemp = 4,linescompare = 0,lines2 = 0):
	lines = np.array(lines)
	lines = lines/1000
	lines2 = np.array(lines2)
	lines2 = lines2/1000
	if physicaltemp == 300:
		propcons = TRL.getCPWPropCons(freqs,w=24e-6, s=13e-6, ep_r=11.9, t=200e-9, rho=2.27e-8) #300K
	elif physicaltemp == 4:
		propcons = TRL.getCPWPropCons(freqs,w=24e-6, s=13e-6, ep_r=11.42, t=200e-9, rho=0.022e-8) #4K
	(alphasigma,betasigma) = TRL.getCalConErrors(freqs,lines,propcons)
	if linescompare == 1:
		(alphasigma2,betsigma2) = TRL.getCalConErrors(freqs,lines2,propcons)
	phases = TRL.getEffPhases(freqs,propcons,lines)
	(alphasigmas,betasigmas) = TRL.getAllCalConErrors(freqs,lines,propcons)
	(alphavar,betavar) = TRL.getAveVariance(alphasigma,betasigma)

	print("Ave. Alpha Variance : " + str(alphavar))
	print("Ave. Beta Variance : " + str(betavar))

	#Plot it All
	fig1 = plt.figure(1,figsize=(14,9))
	fig1.subplots_adjust(hspace =0.3,wspace=0.3)
	ax11 = fig1.add_subplot(221)
	ax11.set_xlabel('Freq (GHz)')
	ax11.set_ylabel('Phase Difference (deg)')
	ax21 = fig1.add_subplot(222)
	ax21.set_xlabel('Freq (GHz)')
	ax21.set_ylabel(r'Im[$\gamma$] (deg/mm)')
	ax22 = ax21.twinx()
	ax22.set_ylabel(r'Re[$\gamma$] (dB/mm)')
	ax31 = fig1.add_subplot(223)
	ax31.set_xlabel('Freq (GHz)')
	ax31.set_ylabel(r'$\sigma_\alpha$')
	ax31.set_yscale('log')
	ax31.set_ylim((0.3,20))
	ax41 = fig1.add_subplot(224)
	ax41.set_xlabel('Freq (GHz)')
	ax41.set_ylabel(r'$\sigma_\beta$')
	ax41.set_yscale('log')
	ax41.set_ylim((0.3,20))

	colors = [ cm.brg(x) for x in np.linspace(0.01, 0.99, len(phases[:,0])) ]

	for i in range(0,len(phases[:,0])):
		ax11.plot(freqs,phases[i,:],color=colors[i])
	ax22.plot(freqs,20/np.log(10)*np.real(propcons)/1000,'-b')
	ax21.plot(freqs,180/math.pi*np.imag(propcons)/1000,'-r')
	ax31.plot(freqs,alphasigma,'-k',linewidth = 2)
	for i in range(0,len(alphasigmas[:,0])):
		ax31.plot(freqs,alphasigmas[i,:],color=colors[i],linestyle = '--')
	ax41.plot(freqs,betasigma,'-k',linewidth = 2)
	for i in range(0,len(betasigmas[:,0])):
		ax41.plot(freqs,betasigmas[i,:],color=colors[i],linestyle = '--')

	if linescompare == 1:	
		fig2 = plt.figure(2)
		ax12 = fig2.add_subplot(111)
		ax12.set_xlabel('Freq (GHz)')
		ax12.set_ylabel(r'$\sigma_\alpha$')
		ax12.plot(freqs[0],alphasigma[0],'-k',linewidth = 2, label = '% Diff')
		ax12.plot(freqs,alphasigma,'-r',linewidth = 2,label = 'Optimized')
		ax12.plot(freqs,alphasigma2,'-b',linewidth = 2, label = 'Starting')
		#ax12.set_xscale('log')
		ax12.set_yscale('log')
		ax122 = ax12.twinx()
		ax122.plot(freqs,100*(alphasigma-alphasigma2)/alphasigma,'-k',linewidth = 2)
		ax122.set_ylabel('% difference')
		ax12.legend(loc='upper right')

	plt.show()
	return 0