#!/usr/bin/python
import numpy as np
import math
import matplotlib
from matplotlib import pyplot as plt
import pylab
import matplotlib.gridspec as gridspec
import TRL_optimize
import TRL_optplot
import plot_ML_TRL

#Basic Option Flags
runopt = 0        #1 to run optimization
runoptplot = 0    #1 to run optimization progression plotting
runplot = 1       #1 to run plotting of a optimal line set, if used with running optimization must specify your lines below

#Optimization Options
specifylines = 1  #0 to use autostarting in optimization or 1 to plot/start with a specific sets of lines
charwidth = 0.5   #characteristic width of parameter variation to start
numsteps = 10000  #number of steps in simulated annealing
initialT = 0.5    #initial optimization "temperature" in terms of your cost function

#Physical Temperature during Cal
physicaltemp = 300  #the physical temperature you want to optimize / plot at

lowfreq = 5
highfreq = 50

#optimization filename to write to / read from for plotting
optfilename = "C:\\Users\\Jake\\Documents\\VNACal\\ML_TRL_test.txt"


#band to optimize over
freqs = np.linspace(lowfreq,highfreq,101)

#build opt array are [name, start temp, numsteps, physicaltemp]
optarray = ['Inverse Variance Sum', initialT, numsteps, physicaltemp]

#build param array #params are [name, start, delta, range]
if specifylines == 1 or (runplot == 1 and runopt == 0): # Specify your own line set here

	print("Using user specified lines")
	#Use this if you want to start from a known set of lines
	lines = [0.2,3,9]
	maxlinelength = 10 #longest line you're willing to have

	lineparams = []
	for i in range(0,len(lines)):
		linename = "L" + str(i+1) + " (mm)"
		if i > 0:# and i < len(lines)-1:
			lineparams.append([linename,lines[i],charwidth,[0,maxlinelength]])
		else:
			lineparams.append([linename,lines[0],0,[0,maxlinelength]])
	print(lineparams)
	
elif specifylines == 0: #Let this algorithm guess your starting line set
	
	lzero = 0.2 #thru line length
	lN = 10  #longest line you're willing to have
	numlines = 2 #number of lines not including thru
	deltazero = 0.4 # in mm, ideally should be set such that deltazero is 90 degrees at upper end of band
	deltadelta = 2*(lN-lzero-numlines*deltazero)/(numlines**2-numlines)
	print("DeltaZero = " + str(deltazero) + ", DeltaDelta = " + str(deltadelta))
	
	lineparams = []
	for i in range(0,numlines+1):
		linename = "L" + str(i+1) + " (mm)"
		if i > 0:# and i < len(lines)-1:
			lineparams.append([linename,lzero + i*deltazero + 0.5*(i**2-i)*deltadelta,charwidth,[0,lN]])
		else:
			lineparams.append([linename,lzero,0,[0,lN]])
	print(lineparams)

if runopt == 1:

	print("Running Simulated Annealing Optimization")
	
	bestparams = TRL_optimize.TRLopt(lineparams, freqs, optarray, optfilename)

	print("Simulated Annealing Finished, Optimal Params are:")
	print(bestparams)

if runoptplot == 1 or runopt == 1:

	print("Running Plotting Code")
	
	TRL_optplot.TRLoptplot(lineparams,optarray,optfilename)
	
if runplot == 1 or runopt == 1:

	if lowfreq <= 1:
		freqs = np.linspace(lowfreq,highfreq,1001)
	else:
		freqs = np.linspace(1,highfreq,1001)
	
	startinglines = []
	for i in range(0,len(lineparams)):
		startinglines.append(lineparams[i][1])
	print(startinglines)
	
	if runopt == 0:
		plot_ML_TRL.plotMLTRL(startinglines,freqs,physicaltemp)
		
	else:
		plot_ML_TRL.plotMLTRL(bestparams,freqs,physicaltemp,1,startinglines)
		
	
	
	

	



